-- SPDX-License-Identifier: CC0-1.0

let Genode = env:DHALL_GENODE

in  { config =
        Genode.Init::{
        , verbose = True
        , children =
            toMap
              { noux =
                  Genode.Init.Start::{
                  , binary = "noux"
                  , exitPropagate = True
                  , resources = { caps = 500, ram = Genode.units.MiB 10 }
                  , routes = [ Genode.ServiceRoute.parent "Timer" ]
                  , config =
                      Some
                        ( Genode.Prelude.XML.text
                            ''
                            <config stdin="/script" stdout="/dev/log" stderr="/dev/log">
                            	<fstab>
                            		<tar name="bash-minimal.tar" />
                            		<dir name="dev"> <log/> <null/> <zero/> </dir>
                            		<dir name="tmp"> <ram /> </dir>
                            <inline name="script">
                            echo "hello world"
                            </inline>
                            	</fstab>
                            	<start name="/bin/bash">
                            		<env name="TERM" value="screen" />
                            	</start>
                            </config>
                            ''
                        )
                  }
              }
        }
    , rom =
        let manifest = env:MANIFEST
        
        in  Genode.Boot.toRomPaths
              [ manifest.bash-minimal.tar.bash-minimal
              , manifest.libc.lib.libc
              , manifest.libc.lib.libm
              , manifest.ncurses.lib.ncurses
              , manifest.noux.bin.noux
              , manifest.noux.lib.libc_noux
              , manifest.os.lib.vfs
              , manifest.posix.lib.posix
              ]
    }
